@@ -1,5 +1,14 @@
 # -*- coding: utf-8 -*-
 
+"""
+Diverse color mathematical functions.
+
+Note:
+In most cases, unless otherwise stated RGB is R'G'B' (gamma-compressed)
+
+"""
+
+import colorsys
 import logging
 import math
 import sys
@@ -106,7 +115,7 @@ def specialpow(a, b, slope_limit=0):
 		v = (max(a ** (1.0 / SMPTE2084_M2) - SMPTE2084_C1, 0) /
 			 (SMPTE2084_C2 - SMPTE2084_C3 * a ** (1.0 / SMPTE2084_M2))) ** (1.0 / SMPTE2084_M1)
 	else:
-		raise ValueError("Invalid gamma %s" % b)
+		raise ValueError("Invalid gamma %r" % b)
 	return v * signScale
 
 
@@ -145,6 +154,99 @@ def DICOM(j, inverse=False):
 				 h * math.pow(logj, 4) + k * math.pow(logj, 5)))
 
 
+class HLG(object):
+	"""
+	Hybrid Log Gamma (HLG) as defined in Rec BT.2100
+	
+	"""
+
+	def __init__(self, black_cdm2=0.0, white_cdm2=1000.0, system_gamma=1.2,
+				 rgb_space="Rec. 2020"):
+		self.black_cdm2 = black_cdm2
+		self.white_cdm2 = white_cdm2
+		self.rgb_space = get_rgb_space(rgb_space)
+		self.system_gamma = system_gamma
+		
+	def oetf(self, v, inverse=False, normalize=1, apply_system_gamma=True,
+			 apply_black_offset=True):
+		"""
+		Hybrid Log Gamma (HLG) OETF
+		
+		"""
+		if normalize == 1:
+			# Normalized to 0..1 range
+			c = 1.00429347
+			d = 1.0
+			e = 3.0
+			k = 1.0 / 12
+			r = 1.0
+		elif normalize == 12:
+			# Normalized to 0..12 range
+			c = 0.55991073
+			d = 4.0
+			e = 1.0
+			k = 1.0
+			r = 0.5
+		else:
+			raise ValueError("Cannot normalize to %s" % normalize)
+		a = 0.17883277
+		b = 0.02372241 * normalize
+		if inverse:
+			# Inverse OETF, RGB -> XYZ
+			E_ = v
+			if 0 <= E_ <= 0.5:
+				E = d * E_ ** 2 / e
+			else:
+				E = math.exp((E_ - c) / a) + b
+			v = E
+			if apply_system_gamma:
+				gamma = self.system_gamma + 0.42 * math.log10(self.white_cdm2 / 1000.0)
+				v = v ** gamma
+			if apply_black_offset:
+				LB = self.black_cdm2 / self.white_cdm2
+				alpha = (1.0 - LB)
+				v = alpha * v + LB
+		else:
+			# OETF, XYZ -> RGB
+			if apply_system_gamma:
+				gamma = self.system_gamma + 0.42 * math.log10(self.white_cdm2 / 1000.0)
+				v = (v / normalize) ** (1 / gamma) * normalize
+			E = v
+			if 0 <= E <= k:
+				E_ = r * math.sqrt(e * E)
+			else:
+				E_ = a * math.log(E - b) + c
+			v = E_
+		return min(v, normalize)
+
+	def eotf(self, R, G, B):
+		"""
+		Hybrid Log Gamma (HLG) EOTF
+
+		Return luminance of displayed components (in cd/m2) or XYZ
+		(normalized to 0..1) if rgb_space is given
+		
+		"""
+		RGB_S = tuple(self.oetf(v, True, 1, False, False) for v in (R, G, B))
+		gamma = self.system_gamma + 0.42 * math.log10(self.white_cdm2 / 1000.0)
+		if self.rgb_space:
+			X, Y, Z = self.rgb_space[-1] * RGB_S
+			if Y:
+				# Apply system gamma
+				Yy = Y ** gamma
+				X, Y, Z = (v / Y * Yy for v in (X, Y, Z))
+			# Apply black offset
+			beta = self.black_cdm2 / self.white_cdm2
+			bp = [E * beta for E in get_whitepoint(self.rgb_space[1])]
+			X, Y, Z = blend_blackpoint(X, Y, Z, (0, 0, 0), bp)
+			return X, Y, Z
+		else:
+			alpha = (self.white_cdm2 - self.black_cdm2)
+			# Rec. 2020 relative luminance coefficients
+			YS = 0.2627 * RGB_S[0] + 0.6780 * RGB_S[1] + 0.0593 * RGB_S[2]
+			return alpha * YS ** gamma + self.black_cdm2
+
+
 rgb_spaces = {
 	# http://brucelindbloom.com/WorkingSpaceInfo.html
 	# ACES: https://github.com/ampas/aces-dev/blob/master/docs/ACES_1.0.1.pdf?raw=true
@@ -156,6 +258,7 @@ rgb_spaces = {
 	# name              gamma             white                     primaries
 	#                                     point                     Rx      Ry      RY          Gx      Gy      GY          Bx      By      BY
 	"ACES":             (1.0,             (0.95265, 1.0, 1.00883), (0.7347, 0.2653, 0.343961), (0.0000, 1.0000, 0.728164), (0.0001,-0.0770,-0.072125)),
+	"ACEScg":           (1.0,             (0.95265, 1.0, 1.00883), (0.7130, 0.2930, 0.272230), (0.1650, 0.8300, 0.674080), (0.1280, 0.0440, 0.053690)),
 	"Adobe RGB (1998)": (2 + 51 / 256.0,  "D65",                   (0.6400, 0.3300, 0.297361), (0.2100, 0.7100, 0.627355), (0.1500, 0.0600, 0.075285)),
 	"Apple RGB":        (1.8,             "D65",                   (0.6250, 0.3400, 0.244634), (0.2800, 0.5950, 0.672034), (0.1550, 0.0700, 0.083332)),
 	"Best RGB":         (2.2,             "D50",                   (0.7347, 0.2653, 0.228457), (0.2150, 0.7750, 0.737352), (0.1300, 0.0350, 0.034191)),
@@ -163,6 +266,7 @@ rgb_spaces = {
 	"Bruce RGB":        (2.2,             "D65",                   (0.6400, 0.3300, 0.240995), (0.2800, 0.6500, 0.683554), (0.1500, 0.0600, 0.075452)),
 	"CIE RGB":          (2.2,             "E",                     (0.7350, 0.2650, 0.176204), (0.2740, 0.7170, 0.812985), (0.1670, 0.0090, 0.010811)),
 	"ColorMatch RGB":   (1.8,             "D50",                   (0.6300, 0.3400, 0.274884), (0.2950, 0.6050, 0.658132), (0.1500, 0.0750, 0.066985)),
+#	"DCDM X'Y'Z'":      (2.6,             "E",                     (1.0000, 0.0000, 0.000000), (0.0000, 1.0000, 1.000000), (0.0000, 0.0000, 0.000000)),
 	"DCI P3":           (2.6,             (0.89459, 1.0, 0.95442), (0.6800, 0.3200, 0.209475), (0.2650, 0.6900, 0.721592), (0.1500, 0.0600, 0.068903)),
 	"Don RGB 4":        (2.2,             "D50",                   (0.6960, 0.3000, 0.278350), (0.2150, 0.7650, 0.687970), (0.1300, 0.0350, 0.033680)),
 	"ECI RGB":          (1.8,             "D50",                   (0.6700, 0.3300, 0.320250), (0.2100, 0.7100, 0.602071), (0.1400, 0.0800, 0.077679)),
@@ -229,10 +333,10 @@ def LMS_wp_adaption_matrix(whitepoint_source=None,
 	XYZWD = get_whitepoint(whitepoint_destination)
 	if XYZWS[1] <= 1.0 and XYZWD[1] > 1.0:
 		# make sure the scaling is identical
-		XYZWS = [v * 100 for v in XYZWS]
+		XYZWD = [v / XYZWD[1] * XYZWS[1] for v in XYZWD]
 	if XYZWD[1] <= 1.0 and XYZWS[1] > 1.0:
 		# make sure the scaling is identical
-		XYZWD = [v * 100 for v in XYZWD]
+		XYZWS = [v / XYZWS[1] * XYZWD[1] for v in XYZWS]
 	Ls, Ms, Ss = XYZ2LMS(XYZWS[0], XYZWS[1], XYZWS[2], cat)
 	Ld, Md, Sd = XYZ2LMS(XYZWD[0], XYZWD[1], XYZWD[2], cat)
 	return Matrix3x3([[Ld/Ls, 0, 0], [0, Md/Ms, 0], [0, 0, Sd/Ss]])
@@ -308,7 +412,7 @@ def blend_blackpoint(X, Y, Z, bp_in=None, bp_out=None, power=40.0):
 	L, a, b = XYZ2Lab(*[v * 100 for v in (X, Y, Z)])
 
 	for bp in (bp_in, bp_out):
-		if not bp:
+		if not bp or tuple(bp) == (0, 0, 0):
 			continue
 		bpLab = XYZ2Lab(*[v * 100 for v in bp])
 		if bp is bp_in:
@@ -374,6 +478,49 @@ def interp(x, xp, fp, left=None, right=None):
 		return fp[lower] + (fp[higher] - fp[lower]) / steps
 
 
+def interp_resize(iterable, new_size, use_numpy=False):
+	""" Change size of iterable through linear interpolation """
+	result = []
+	x_new = range(len(iterable))
+	interp = Interp(x_new, iterable, use_numpy=use_numpy)
+	for i in xrange(new_size):
+		result.append(interp(i / (new_size - 1.0) * (len(iterable) - 1.0)))
+	return result
+
+
+def smooth_avg(values, passes=1, window=None):
+	"""
+	Smooth values (moving average).
+	
+	passses   Number of passes
+	window    Tuple or list containing weighting factors. Its length
+			  determines the size of the window to use.
+			  Defaults to (1.0, 1.0, 1.0)
+	
+	"""
+	if not window or len(window) < 3 or len(window) % 2 != 1:
+		window = (1.0, 1.0, 1.0)
+	for x in xrange(0, passes):
+		data = []
+		for j, v in enumerate(values):
+			tmpwindow = window
+			while j > 0 and j < len(values) - 1 and len(tmpwindow) >= 3:
+				tl = (len(tmpwindow) - 1) / 2
+				# print j, tl, tmpwindow
+				if tl > 0 and j - tl >= 0 and j + tl <= len(values) - 1:
+					windowslice = values[j - tl:j + tl + 1]
+					windowsize = 0
+					for k, weight in enumerate(tmpwindow):
+						windowsize += float(weight) * windowslice[k]
+					v = windowsize / sum(tmpwindow)
+					break
+				else:
+					tmpwindow = tmpwindow[1:-1]
+			data.append(v)
+		values = data
+	return data
+
+
 def compute_bpc(bp_in, bp_out):
 	"""
 	Black point compensation. Implemented as a linear scaling in XYZ. 
@@ -410,7 +557,8 @@ def compute_bpc(bp_in, bp_out):
 	return matrix, offset
 
 
-def delta(L1, a1, b1, L2, a2, b2, method="1976", p1=None, p2=None, p3=None):
+def delta(L1, a1, b1, L2, a2, b2, method="1976", p1=None, p2=None, p3=None,
+		  cie94_use_symmetric_chrominance=True):
 		"""
 		Compute the delta of two samples
 
@@ -454,11 +602,10 @@ def delta(L1, a1, b1, L2, a2, b2, method="1976", p1=None, p2=None, p3=None):
 			SL = 1.0
 			K1 = 0.048 if textiles else 0.045
 			K2 = 0.014 if textiles else 0.015
-			# brucelindbloom.com formula originally used C1 instead of C_,
-			# but the results are different from ProfileMaker/MeasureTool 
-			# implementation, so use this instead
-			# (found on www.farbmetrik-gall.de/cielab/korrcielab/cie94.html)
-			C_ = (math.sqrt((1 + (K1 * C1)) * (1 + (K1 * C2))) - 1) / K1
+			if cie94_use_symmetric_chrominance:
+				C_ = math.sqrt(C1 * C2)
+			else:
+				C_ = C1
 			SC = 1.0 + K1 * C_
 			SH = 1.0 + K2 * C_
 			KL = 2.0 if textiles else 1.0
@@ -551,6 +698,43 @@ def is_similar_matrix(matrix1, matrix2, digits=3):
 	return matrix1.rounded(digits) == matrix2.rounded(digits)
 
 
+def four_color_matrix(XrR, YrR, ZrR, XrG, YrG, ZrG, XrB, YrB, ZrB, XrW, YrW, ZrW,
+					  XmR, YmR, ZmR, XmG, YmG, ZmG, XmB, YmB, ZmB, XmW, YmW, ZmW,
+					  Y_correction=True):
+	"""
+	Four-Color Matrix Method for Correction of Tristimulus Colorimeters
+	
+	Based on paper published in Proc., IS&T Fifth Color Imaging Conference,
+	301-305 (1997) and IS&T Sixth Color Imaging Conference (1998).
+	
+	"""
+	XYZ = locals()
+	xyz = {}
+	M = {}
+	k = {}
+	for s in "mr":
+		xyz[s] = {}
+		for color in "RGBW":
+			X, Y, Z = (XYZ[component + s + color] for component in "XYZ")
+			x, y = XYZ2xyY(X, Y, Z)[:2]
+			xyz[s][color] = x, y, 1 - x - y
+		M[s] = Matrix3x3([xyz[s][color] for color in "RGB"]).transposed()
+		k[s] = M[s].inverted() * xyz[s]["W"]
+		M[s + "RGB"] = M[s] * Matrix3x3([[k[s][0], 0, 0],
+										 [0, k[s][1], 0],
+										 [0, 0, k[s][2]]])
+	R = M["rRGB"] * M["mRGB"].inverted()
+	if Y_correction:
+		# The Y calibration factor kY is obtained as the ratio of the reference
+		# luminance value to the matrix-corrected Y value, as defined in
+		# Four-Color Matrix Method for Correction of Tristimulus Colorimeters –
+		# Part 2
+		MW = XmW, YmW, ZmW
+		kY = YrW / (R * MW)[1]
+		R[:] = [[kY * v for v in row] for row in R]
+	return R
+
+
 def get_gamma(values, scale=1.0, vmin=0.0, vmax=1.0, average=True, least_squares=False):
 	""" Return average or least squares gamma or a list of gamma values """
 	if least_squares:
@@ -586,7 +770,7 @@ def guess_cat(chad, whitepoint_source=None, whitepoint_destination=None):
 	""" Try and guess the chromatic adaption transform used in a chromatic 
 	adaption matrix as found in an ICC profile's 'chad' tag """
 	if chad == [[1, 0, 0], [0, 1, 0], [0, 0, 1]]:
-		return "None"
+		return "XYZ scaling"
 	for cat in cat_matrices:
 		if is_similar_matrix((chad * cat_matrices[cat].inverted() * 
 							  LMS_wp_adaption_matrix(whitepoint_destination, 
@@ -677,12 +861,18 @@ def DIN99bcdLCH2Lab(L99, C99, H99, x, l1, l2, deg, f1, c1, c2, whitepoint=None):
 	H99 -= deg
 	L, a, b = DIN99familyLHCG2Lab(L99, H99, C99, G, 1.0, l1, l2, deg, f1)
 	if x:
-		X, Y, Z = Lab2XYZ(L, a, b, whitepoint, scale=100)
-		X = (X + x * Z) / (1 + x)
+		whitepoint99d = XYZ2DIN99cdXYZ(*get_whitepoint(whitepoint, 100), x=x)
+		X, Y, Z = Lab2XYZ(L, a, b, whitepoint99d, scale=100)
+		X, Y, Z = DIN99cdXYZ2XYZ(X, Y, Z, x)
 		L, a, b = XYZ2Lab(X, Y, Z, whitepoint)
 	return L, a, b
 
 
+def DIN99cdXYZ2XYZ(X, Y, Z, x):
+	X = (X + x * Z) / (1 + x)
+	return X, Y, Z
+
+
 def DIN99familyLHCG2Lab(L99, H99, C99, G, kE, l1, l2, deg, f1):
 	L = (math.exp((L99 * kE) / l1) - 1) / l2
 	h99ef = H99 * math.pi / 180
@@ -718,42 +908,12 @@ def DIN99familyab2DIN99CH(a99, b99):
 	return C99, H99
 
 
+def HSL2RGB(H, S, L, scale=1.0):
+    return tuple(v * scale for v in colorsys.hls_to_rgb(H, L, S))
+
+
 def HSV2RGB(H, S, V, scale=1.0):
-	if S == 0:
-		return (V * scale, ) * 3
-	else:
-		h = H * 6.0
-		if h == 6:
-			h = 0
-		i = int(h)
-		component_1 = V * (1.0 - S)
-		component_2 = V * (1.0 - S * (h - i))
-		component_3 = V * (1.0 - S * (1.0 - (h - i)))
-		if i == 0:
-			R = V
-			G = component_3
-			B = component_1
-		elif i == 1:
-			R = component_2
-			G = V
-			B = component_1
-		elif i == 2:
-			R = component_1
-			G = V
-			B = component_3
-		elif i == 3:
-			R = component_1
-			G = component_2
-			B = V
-		elif i == 4:
-			R = component_3
-			G = component_1
-			B = V
-		else:
-			R = V
-			G = component_1
-			B = component_2
-		return R * scale, G * scale, B * scale
+	return tuple(v * scale for v in colorsys.hsv_to_rgb(H, S, V))
 
 
 def get_DBL_MIN():
@@ -858,12 +1018,13 @@ def Lab2Luv(L, a, b, whitepoint=None, scale=100):
 
 
 def Lab2RGB(L, a, b, rgb_space=None, scale=1.0, round_=False, clamp=True,
-			whitepoint=None, noadapt=False, cat="Bradford"):
+			whitepoint=None, whitepoint_source=None, noadapt=False,
+			cat="Bradford"):
 	""" Convert from Lab to RGB """
 	X, Y, Z = Lab2XYZ(L, a, b, whitepoint)
 	if not noadapt:
-		X, Y, Z = adapt(X, Y, Z, whitepoint, rgb_space[1] if rgb_space
-											 else "D65", cat)
+		X, Y, Z = adapt(X, Y, Z, whitepoint_source, rgb_space[1] if rgb_space
+													else "D65", cat)
 	return XYZ2RGB(X, Y, Z, rgb_space, scale, round_, clamp)
 
 
@@ -977,60 +1138,12 @@ def RGB2HSI(R, G, B, scale=1.0):
 
 
 def RGB2HSL(R, G, B, scale=1.0):
-	RGB_min = min(R, G, B)
-	RGB_max = max(R, G, B)
-	delta = RGB_max - RGB_min
-	L = (RGB_max + RGB_min) / 2.0
-	if delta == 0:
-		# Gray
-		H = 0
-		S = 0
-	else:
-		if L < .5:
-			S = delta / (RGB_max + RGB_min)
-		else:
-			S = delta / (2.0 - RGB_max - RGB_min)
-		delta_R = (((RGB_max - R) / 6.0) + (delta / 2.0) ) / delta
-		delta_G = (((RGB_max - G) / 6.0) + (delta / 2.0) ) / delta
-		delta_B = (((RGB_max - B) / 6.0) + (delta / 2.0) ) / delta
-		if R == RGB_max:
-			H = delta_B - delta_G
-		elif G == RGB_max:
-			H = (1.0 / 3.0) + delta_R - delta_B
-		elif B == RGB_max:
-			H = (2.0 / 3.0) + delta_G - delta_R
-		if H < 0:
-			H += 1.0
-		if H > 1:
-			H -= 1.0
-	return H * scale, S * scale, L * scale
+	H, L, S = colorsys.rgb_to_hls(R, G, B)
+	return tuple(v * scale for v in (H, S, L))
 
 
 def RGB2HSV(R, G, B, scale=1.0):
-	RGB_min = min( R, G, B )
-	RGB_max = max( R, G, B )
-	delta = RGB_max - RGB_min
-	V = RGB_max
-	if ( delta == 0 ):
-		# Gray
-		H = 0
-		S = 0
-	else:
-		S = delta / RGB_max
-		delta_R = (((RGB_max - R) / 6.0) + (delta / 2.0) ) / delta
-		delta_G = (((RGB_max - G) / 6.0) + (delta / 2.0) ) / delta
-		delta_B = (((RGB_max - B) / 6.0) + (delta / 2.0) ) / delta
-		if R == RGB_max:
-			H = delta_B - delta_G
-		elif G == RGB_max:
-			H = (1.0 / 3.0) + delta_R - delta_B
-		elif B == RGB_max:
-			H = (2.0 / 3.0) + delta_G - delta_R
-		if H < 0:
-			H += 1.0
-		if H > 1:
-			H -= 1.0
-	return H * scale, S * scale, V * scale
+	return tuple(v * scale for v in colorsys.rgb_to_hsv(R, G, B))
 
 
 def RGB2ICtCp(R, G, B, rgb_space="Rec. 2020 ST2084"):
@@ -1137,6 +1250,83 @@ def RGB2XYZ(R, G, B, rgb_space=None, scale=1.0):
 	return tuple(v * scale for v in XYZ)
 
 
+def RGB2YCbCr(R, G, B, rgb_space="NTSC 1953", bits=8):
+	""" R'G'B' to Y'CbCr quantized to n bits """
+	return YPbPr2YCbCr(*RGB2YPbPr(R, G, B, rgb_space), bits=bits)
+
+
+def RGB2YPbPr(R, G, B, rgb_space="NTSC 1953"):
+	""" R'G'B' to Y'PbPr """
+	return RGB2YPbPr_matrix(rgb_space) * (R, G, B)
+
+
+def RGB2YPbPr_matrix(rgb_space="NTSC 1953"):
+	(trc, whitepoint, (rx, ry, rY), (gx, gy, gY), (bx, by, bY),
+	 matrix) = get_rgb_space(rgb_space)
+	if matrix == get_rgb_space("NTSC 1953")[-1]:
+		ndigits = 3
+	else:
+		ndigits = 4
+	KR = round(rY, ndigits)
+	KB = round(bY, ndigits)
+	KG = 1.0 - KR - KB
+	Pb_scale = ((1 - KB) / 0.5)
+	Pr_scale = ((1 - KR) / 0.5)
+	return Matrix3x3([[KR, KG, KB],
+					  [-KR / Pb_scale, -KG / Pb_scale, 0.5],
+					  [0.5, -KG / Pr_scale, -KB / Pr_scale]])
+
+
+def YCbCr2YPbPr(Y, Cb, Cr, bits=8):
+	""" Y'CbCr to Y'PbPr """
+	bitlevels = 2 ** bits
+	Yblack = 16 / 256.0 * bitlevels
+	Yscale = 219 / 256.0 * bitlevels
+	Y -= Yblack
+	Y /= Yscale
+	Cneutral = 128 / 256.0 * bitlevels
+	Cscale = 224 / 256.0 * bitlevels
+	Cb -= Cneutral
+	Cb /= Cscale
+	Cr -= Cneutral
+	Cr /= Cscale
+	return Y, Cb, Cr
+
+
+def YCbCr2RGB(Y, Cb, Cr, rgb_space="NTSC 1953", bits=8, scale=1.0, round_=False,
+			  clamp=True):
+	""" Y'CbCr to R'G'B' """
+	Y, Pb, Pr = YCbCr2YPbPr(Y, Cb, Cr, bits)
+	return YPbPr2RGB(Y, Pb, Pr, rgb_space, scale, round_, clamp)
+
+
+def YPbPr2RGB(Y, Pb, Pr, rgb_space="NTSC 1953", scale=1.0, round_=False,
+			  clamp=True):
+	""" Y'PbPr to R'G'B' """
+	RGB = RGB2YPbPr_matrix(rgb_space).inverted() * (Y, Pb, Pr)
+	for i in xrange(3):
+		if clamp:
+			RGB[i] = min(1.0, max(0.0, RGB[i]))
+		RGB[i] *= scale
+		if round_ is not False:
+			RGB[i] = round(RGB[i], round_)
+	return RGB
+
+
+def YPbPr2YCbCr(Y, Pb, Pr, bits=8):
+	""" Y'PbPr' to Y'CbCr quantized to n bits """
+	bitlevels = 2 ** bits
+	Yblack = 16 / 256.0 * bitlevels
+	Yscale = 219 / 256.0 * bitlevels
+	Y = Yblack + Yscale * Y
+	Cneutral = 128 / 256.0 * bitlevels
+	Cscale = 224 / 256.0 * bitlevels
+	Cb = Cneutral + Cscale * Pb
+	Cr = Cneutral + Cscale * Pr
+	Y, Cb, Cr = (int(round(v)) for v in (Y, Cb, Cr))
+	return Y, Cb, Cr
+
+
 def RGBsaturation(R, G, B, saturation, rgb_space=None):
 	""" (De)saturate a RGB color in CIE xy and return the RGB and xyY values """
 	whitepoint = RGB2XYZ(1, 1, 1, rgb_space=rgb_space)
@@ -1158,6 +1348,12 @@ def xyYsaturation(x, y, Y, wx, wy, saturation):
 	return wx + (x - wx) * saturation,  wy + (y - wy) * saturation, Y
 
 
+def convert_range(v, oldmin=0, oldmax=1, newmin=0, newmax=1):
+	oldrange = float(oldmax - oldmin)
+	newrange = newmax - newmin
+	return (((v - oldmin) * newrange) / oldrange) + newmin
+
+
 def rgb_to_xyz_matrix(rx, ry, gx, gy, bx, by, whitepoint=None, scale=1.0):
 	""" Create and return an RGB to XYZ matrix. """
 	whitepoint = get_whitepoint(whitepoint, scale)
@@ -1221,24 +1417,32 @@ def get_standard_illuminant(illuminant_name="D50",
 get_standard_illuminant.cache = {}
 
 
-def get_whitepoint(whitepoint=None, scale=1.0):
+def get_whitepoint(whitepoint=None, scale=1.0, planckian=False):
 	""" Return a whitepoint as XYZ coordinates """
 	if isinstance(whitepoint, (list, tuple)):
 		return whitepoint
 	if not whitepoint:
 		whitepoint = "D50"
-	cachehash = whitepoint, scale
+	cachehash = whitepoint, scale, planckian
 	cache = get_whitepoint.cache.get(cachehash, None)
 	if cache:
 		return cache
 	if isinstance(whitepoint, basestring):
 		whitepoint = get_standard_illuminant(whitepoint)
 	elif isinstance(whitepoint, (float, int)):
-		whitepoint = CIEDCCT2XYZ(whitepoint)
+		cct = whitepoint
+		if planckian:
+			whitepoint = planckianCT2XYZ(cct)
+			if not whitepoint:
+				raise ValueError("Planckian color temperature %i out of range "
+								 "(1667, 25000)" % cct)
+		else:
+			whitepoint = CIEDCCT2XYZ(cct)
+			if not whitepoint:
+				raise ValueError("Daylight color temperature %i out of range "
+								 "(4000, 25000)" % cct)
 	if scale > 1.0 and whitepoint[1] == 100:
 		scale = 1.0
-	if whitepoint[1] * scale > 100:
-		ValueError("Y value out of range after scaling: %s" % (whitepoint[1] * scale))
 	whitepoint = tuple(v * scale for v in whitepoint)
 	get_whitepoint.cache[cachehash] = whitepoint
 	return whitepoint
@@ -1247,6 +1451,54 @@ def get_whitepoint(whitepoint=None, scale=1.0):
 get_whitepoint.cache = {}
 
 
+def make_monotonically_increasing(iterable, passes=0, window=None):
+	"""
+	Given an iterable or sequence, make the values monotonically increasing
+	by linear interpolation.
+	
+	If iterable is a dict, keep the keys of the original.
+	
+	If passes is non-zero, apply moving average smoothing to the values
+	before making them monotonically increasing.
+	
+	"""
+	if isinstance(iterable, dict):
+		keys = iterable.keys()
+		values = iterable.values()
+	else:
+		if hasattr(iterable, "next"):
+			values = list(iterable)
+		else:
+			values = iterable
+		keys = xrange(len(values))
+	if passes:
+		values = smooth_avg(values, passes, window)
+	sequence = zip(keys, values)
+	numvalues = len(sequence)
+	i = 0
+	while len(sequence) > i and i < len(sequence) - 2:
+		# Do not remove the last value if it identical to the second to last
+		while len(sequence) > i + 1 and sequence[i + 1][1] <= sequence[i][1]:
+			# Decreasing or identical
+			sequence.pop(i + 1)
+		i += 1
+	if len(sequence) > 1:
+		# Remove the second to last value if it is identical to the last
+		if sequence[-2][1] >= sequence[-1][1]:
+			# Decreasing or identical
+			sequence.pop(-2)
+	# Interpolate to original size
+	x_new = [item[0] for item in sequence]
+	y = [item[1] for item in sequence]
+	values = []
+	for i in xrange(numvalues):
+		values.append(interp(i, x_new, y))
+	if isinstance(iterable, dict):
+		# Add in original keys
+		return iterable.__class__(zip(keys, values))
+	return values
+
+
 def planckianCT2XYZ(T, scale=1.0):
 	""" Convert from planckian temperature to XYZ.
 	
@@ -1331,8 +1583,8 @@ def xyY2XYZ(x, y, Y=1.0):
 	"""
 	if y == 0:
 		return 0, 0, 0
-	X = (x * Y) / y
-	Z = ((1 - x - y) * Y) / y
+	X = float(x * Y) / y
+	Z = float((1 - x - y) * Y) / y
 	return X, Y, Z
 
 
@@ -1461,11 +1713,17 @@ def XYZ2DIN99cd(X, Y, Z, x, l1, l2, deg, f1, c1, c2, whitepoint=None):
 
 
 def XYZ2DIN99cdLCH(X, Y, Z, x, l1, l2, deg, f1, c1, c2, whitepoint=None):
-	X = (1 + x) * X - x * Z
-	L, a, b = XYZ2Lab(X, Y, Z, whitepoint)
+	X, Y, Z = XYZ2DIN99cdXYZ(X, Y, Z, x)
+	whitepoint99d = XYZ2DIN99cdXYZ(*get_whitepoint(whitepoint, 100), x=x)
+	L, a, b = XYZ2Lab(X, Y, Z, whitepoint99d)
 	return Lab2DIN99bcdLCH(L, a, b, l1, l2, deg, f1, c1, c2)
 
 
+def XYZ2DIN99cdXYZ(X, Y, Z, x):
+	X = (1 + x) * X - x * Z
+	return X, Y, Z
+
+
 def XYZ2DIN99d(X, Y, Z, whitepoint=None):
 	return XYZ2DIN99cd(X, Y, Z, .12, 325.221, .0036, 50, 1.14, 22.5, .06,
 					   whitepoint)
@@ -1476,6 +1734,29 @@ def XYZ2DIN99dLCH(X, Y, Z, whitepoint=None):
 						  whitepoint)
 
 
+def XYZ2IPT(X, Y, Z):
+	XYZ2LMS_matrix = get_cat_matrix("IPT")
+	LMS = XYZ2LMS_matrix * (X, Y, Z)
+	for i, component in enumerate(LMS):
+		if component >= 0:
+			LMS[i] **= 0.43
+		else:
+			LMS[i] = -(-component) ** 0.43
+	return LMS2IPT_matrix * LMS
+
+
+def IPT2XYZ(I, P, T):
+	XYZ2LMS_matrix = get_cat_matrix("IPT")
+	LMS2XYZ_matrix = XYZ2LMS_matrix.inverted()
+	LMS = IPT2LMS_matrix * (I, P, T)
+	for i, component in enumerate(LMS):
+		if component >= 0:
+			LMS[i] **= 1 / 0.43
+		else:
+			LMS[i] = -(-component) ** (1 / 0.43)
+	return LMS2XYZ_matrix * LMS
+
+
 def XYZ2Lab(X, Y, Z, whitepoint=None):
 	"""
 	Convert from XYZ to Lab.
@@ -1505,6 +1786,92 @@ def XYZ2Lab(X, Y, Z, whitepoint=None):
 	return L, a, b
 
 
+def XYZ2Lpt(X, Y, Z, whitepoint=None):
+	"""
+	Convert from XYZ to Lpt
+	
+	This is a modern update to L*a*b*, based on IPT space.
+	
+	Differences to L*a*b* and IPT:
+	- Using inverse CIE 2012 2degree LMS to XYZ matrix instead of
+	  Hunt-Pointer-Estevez Von Kries chromatic adapation in LMS space.
+	- Using L* compression rather than IPT pure 0.43 power.
+	- Tweaked LMS' to IPT matrix to account for change in XYZ to LMS matrix.
+	- Output scaled to L*a*b* type ranges, to maintain 1 JND scale.
+	- L* value is not a non-linear Y value.
+	
+	The input Y value needs to be in the nominal range [0.0, 100.0] and 
+	other input values scaled accordingly.
+	The output L value is in the nominal range [0.0, 100.0].
+	
+	whitepoint can be string (e.g. "D50"), a tuple of XYZ coordinates or 
+	color temperature as float or int. Defaults to D50 if not set.
+	
+	"""
+	# Adapted from Argyll/icc/icc.c
+
+	xyz2lms = get_cat_matrix("CIE2012_2")
+	
+	wlms = xyz2lms * get_whitepoint(whitepoint, 100)
+
+	lms = xyz2lms * (X, Y, Z)
+
+	for j in xrange(3):
+		lms[j] /= wlms[j]
+
+		if (lms[j] > 0.008856451586):
+			lms[j] = pow(lms[j], 1.0 / 3.0);
+		else:
+			lms[j] = 7.787036979 * lms[j] + 16.0 / 116.0
+		lms[j] = 116.0 * lms[j] - 16.0
+
+	return LMS2Lpt_matrix * lms
+
+
+def Lpt2XYZ(L, p, t, whitepoint=None, scale=1.0):
+	"""
+	Convert from Lpt to XYZ
+	
+	This is a modern update to L*a*b*, based on IPT space.
+	
+	Differences to L*a*b* and IPT:
+	- Using inverse CIE 2012 2degree LMS to XYZ matrix instead of
+	  Hunt-Pointer-Estevez Von Kries chromatic adapation in LMS space.
+	- Using L* compression rather than IPT pure 0.43 power.
+	- Tweaked LMS' to IPT matrix to account for change in XYZ to LMS matrix.
+	- Output scaled to L*a*b* type ranges, to maintain 1 JND scale.
+	- L* value is not a non-linear Y value.
+	
+	The input L* value needs to be in the nominal range [0.0, 100.0] and 
+	other input values scaled accordingly.
+	The output XYZ values are in the nominal range [0.0, 1.0].
+	
+	whitepoint can be string (e.g. "D50"), a tuple of XYZ coordinates or 
+	color temperature as float or int. Defaults to D50 if not set.
+	
+	"""
+	# Adapted from Argyll/icc/icc.c
+
+	xyz2lms = get_cat_matrix("CIE2012_2")
+	lms2xyz = xyz2lms.inverted()
+
+	wlms = xyz2lms * get_whitepoint(whitepoint, scale)
+
+	lms = Lpt2LMS_matrix * (L, p, t)
+
+	for j in xrange(3):
+		lms[j] = (lms[j] + 16.0) / 116.0
+
+		if lms[j] > 24.0 / 116.0:
+			lms[j] = pow(lms[j], 3.0)
+		else:
+			lms[j] = (lms[j] - 16.0 / 116.0) / 7.787036979
+
+		lms[j] *= wlms[j]
+
+	return lms2xyz * lms
+
+
 def XYZ2Lu_v_(X, Y, Z, whitepoint=None):
 	""" Convert from XYZ to CIE Lu'v' """
 
@@ -1627,8 +1994,8 @@ def XYZ2xyY(X, Y, Z, whitepoint=None):
 		# to 0, thus resulting in ZeroDivisionError later
 		x, y, Y = XYZ2xyY(*get_whitepoint(whitepoint))
 		return x, y, 0.0
-	x = X / (X + Y + Z)
-	y = Y / (X + Y + Z)
+	x = X / float(X + Y + Z)
+	y = Y / float(X + Y + Z)
 	return x, y, Y
 
 
@@ -2160,18 +2527,32 @@ class gam_fits(object):
 
 class Interp(object):
 
-	def __init__(self, xp, fp, left=None, right=None):
+	def __init__(self, xp, fp, left=None, right=None, use_numpy=False):
+		if use_numpy:
+			# Use numpy for speed
+			import numpy
+			xp = numpy.array(xp)
+			fp = numpy.array(fp)
+			self.numpy = numpy
 		self.xp = xp
 		self.fp = fp
 		self.left = left
 		self.right = right
 		self.lookup = {}
+		self.use_numpy = use_numpy
 
 	def __call__(self, x):
 		if not x in self.lookup:
-			self.lookup[x] = interp(x, self.xp, self.fp, self.left, self.right)
+			self.lookup[x] = self._interp(x)
 		return self.lookup[x]
 
+	def _interp(self, x):
+		if self.use_numpy:
+			return self.numpy.interp(x, self.xp, self.fp, self.left, self.right)
+		else:
+			return interp(x, self.xp, self.fp, self.left, self.right)
+		
+
 
 class BT1886(object):
 	# Adapted from ArgyllCMS xicc/xicc.c
@@ -2468,7 +2849,26 @@ cat_matrices = {"Bradford": Matrix3x3([[ 0.89510,  0.26640, -0.16140],
 										[ 0.00000,  0.00000,  0.91822]]),
 				"XYZ scaling": Matrix3x3([[1, 0, 0],
 										  [0, 1, 0],
-										  [0, 0, 1]])}
+										  [0, 0, 1]]),
+				"IPT":  Matrix3x3([[ 0.4002, 0.7075, -0.0807],
+								   [-0.2280, 1.1500,  0.0612],
+								   [ 0.0000, 0.0000,  0.9184]]),
+				# Inverse CIE 2012 2deg LMS to XYZ matrix from Argyll/icc/icc.c
+				"CIE2012_2": Matrix3x3([[ 0.2052445519046028,  0.8334486497310412, -0.0386932016356441],
+										[-0.4972221301804286,  1.4034846060306130,  0.0937375241498157],
+										[ 0.0000000000000000,  0.0000000000000000,  1.0000000000000000]])}
+
+LMS2IPT_matrix = Matrix3x3([[ 0.4000,  0.4000,  0.2000],
+							[ 4.4550, -4.8510,  0.3960],
+							[ 0.8056,  0.3572, -1.1628]])
+IPT2LMS_matrix = LMS2IPT_matrix.inverted()
+
+# Tweaked LMS to IPT matrix to account for CIE 2012 2deg XYZ to LMS matrix
+# From Argyll/icc/icc.c
+LMS2Lpt_matrix = Matrix3x3([[ 0.6585034777870502,  0.1424555300344579,  0.1990409921784920],
+							[ 5.6413505933276049, -6.1697985811414187,  0.5284479878138138],
+							[ 1.6370552576322106,  0.0192823194340315, -1.6563375770662419]])
+Lpt2LMS_matrix = LMS2Lpt_matrix.inverted()
 
 standard_illuminants = {
 	# 1st level is the standard name => illuminant definitions
