@@ -19,7 +19,8 @@ from meta import name as appname, version
 from options import debug
 from util_os import islink, readlink, waccess
 from util_str import safe_unicode, strtr
-from worker import Error, Info, get_current_profile_path, show_result_dialog
+from worker import (Error, Info, UnloggedInfo, get_current_profile_path,
+					show_result_dialog)
 import ICCProfile as ICCP
 import colormath
 import config
@@ -145,6 +146,7 @@ class LUT3DFrame(BaseFrame):
 											  self.lut3d_trc_black_output_offset_ctrl_handler)
 		self.lut3d_hdr_peak_luminance_ctrl.Bind(wx.EVT_TEXT,
 											self.lut3d_hdr_peak_luminance_handler)
+		self.lut3d_hdr_display_ctrl.Bind(wx.EVT_CHOICE, self.lut3d_hdr_display_handler)
 		self.lut3d_hdr_maxcll_ctrl.Bind(wx.EVT_TEXT,
 										self.lut3d_hdr_maxcll_handler)
 		self.lut3d_content_colorspace_ctrl.Bind(wx.EVT_CHOICE,
@@ -194,7 +196,10 @@ class LUT3DFrame(BaseFrame):
 							options=("3dlut.", "last_3dlut_path",
 									 "position.lut3dframe", "size.lut3dframe"))
 		if event:
-			event.Skip()
+			# Hide first (looks nicer)
+			self.Hide()
+			# Need to use CallAfter to prevent hang under Windows if minimized
+			wx.CallAfter(self.Destroy)
 	
 	def use_abstract_profile_ctrl_handler(self, event):
 		setcfg("3dlut.use_abstract_profile",
@@ -233,6 +238,7 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_trc_black_output_offset_intctrl.Enable(v)
 		self.lut3d_trc_black_output_offset_intctrl_label.Enable(v)
 		self.lut3d_show_input_value_clipping_warning(event)
+		self.lut3d_show_hdr_display_control()
 
 	def lut3d_show_input_value_clipping_warning(self, layout):
 		self.panel.Freeze()
@@ -251,13 +257,23 @@ class LUT3DFrame(BaseFrame):
 		setcfg("3dlut.output.profile.apply_cal",
 			   int(self.lut3d_apply_cal_cb.GetValue()))
 
+	def lut3d_hdr_display_handler(self, event):
+		if (self.lut3d_hdr_display_ctrl.GetSelection() and
+			not getcfg("3dlut.hdr_display")):
+			if not show_result_dialog(UnloggedInfo(lang.getstr("3dlut.format.madVR.hdr.confirm")),
+									  self, confirm=lang.getstr("ok")):
+				self.lut3d_hdr_display_ctrl.SetSelection(0)
+				return
+		self.lut3d_set_option("3dlut.hdr_display",
+							  self.lut3d_hdr_display_ctrl.GetSelection())
+
 	def lut3d_hdr_peak_luminance_handler(self, event):
-		setcfg("3dlut.hdr_peak_luminance",
-			   self.lut3d_hdr_peak_luminance_ctrl.GetValue())
+		self.lut3d_set_option("3dlut.hdr_peak_luminance",
+							  self.lut3d_hdr_peak_luminance_ctrl.GetValue())
 
 	def lut3d_hdr_maxcll_handler(self, event):
-		setcfg("3dlut.hdr_maxcll",
-			   self.lut3d_hdr_maxcll_ctrl.GetValue())
+		self.lut3d_set_option("3dlut.hdr_maxcll",
+							  self.lut3d_hdr_maxcll_ctrl.GetValue())
 
 	def lut3d_trc_black_output_offset_ctrl_handler(self, event):
 		if event.GetId() == self.lut3d_trc_black_output_offset_intctrl.GetId():
@@ -297,27 +313,26 @@ class LUT3DFrame(BaseFrame):
 			self.lut3d_set_option("3dlut.trc_gamma", 2.4)
 			self.lut3d_set_option("3dlut.trc_gamma_type", "B")
 			self.lut3d_set_option("3dlut.trc_output_offset", 0.0)
-			setcfg("3dlut.trc", "bt1886")
-			self.lut3d_update_trc_controls()
+			trc = "bt1886"
 		elif self.lut3d_trc_ctrl.GetSelection() == 0:
 			# Pure power gamma 2.2
 			self.lut3d_set_option("3dlut.trc_gamma", 2.2)
 			self.lut3d_set_option("3dlut.trc_gamma_type", "b")
 			self.lut3d_set_option("3dlut.trc_output_offset", 1.0)
-			setcfg("3dlut.trc", "gamma2.2")
-			self.lut3d_update_trc_controls()
+			trc = "gamma2.2"
 		elif self.lut3d_trc_ctrl.GetSelection() == 2:  # SMPTE 2084, hard clip
 			self.lut3d_set_option("3dlut.trc_output_offset", 0.0)
-			setcfg("3dlut.trc", "smpte2084.hardclip")
-			setcfg("3dlut.hdr_maxcll", 10000)
-			self.lut3d_update_trc_controls()
+			trc = "smpte2084.hardclip"
+			self.lut3d_set_option("3dlut.hdr_maxcll", 10000)
 		elif self.lut3d_trc_ctrl.GetSelection() == 3:  # SMPTE 2084, roll-off clip
 			self.lut3d_set_option("3dlut.trc_output_offset", 0.0)
-			setcfg("3dlut.trc", "smpte2084.rolloffclip")
-			setcfg("3dlut.hdr_maxcll", 10000)
-			self.lut3d_update_trc_controls()
+			trc = "smpte2084.rolloffclip"
+			self.lut3d_set_option("3dlut.hdr_maxcll", 10000)
 		else:
-			setcfg("3dlut.trc", "customgamma")
+			trc = "customgamma"
+		self.lut3d_set_option("3dlut.trc", trc)
+		if trc != "customgamma":
+			self.lut3d_update_trc_controls()
 		self.lut3d_show_trc_controls()
 		self.Thaw()
 
@@ -336,11 +351,6 @@ class LUT3DFrame(BaseFrame):
 	def lut3d_encoding_input_ctrl_handler(self, event):
 		encoding = self.encoding_input_ab[self.encoding_input_ctrl.GetSelection()]
 		self.lut3d_set_option("3dlut.encoding.input", encoding)
-		if getcfg("3dlut.format") == "eeColor":
-			if encoding == "T":
-				encoding = "t"
-			self.encoding_output_ctrl.SetSelection(self.encoding_output_ba[encoding])
-			self.lut3d_set_option("3dlut.encoding.output", encoding)
 		if getattr(self, "lut3dframe", None):
 			self.lut3dframe.lut3d_update_encoding_controls()
 		elif self.Parent:
@@ -430,8 +440,9 @@ class LUT3DFrame(BaseFrame):
 				else:
 					xyY = rgb_space[2:][i - 1]
 				for j, coord in enumerate("xy"):
-					v = xyY[j]
-					setcfg("3dlut.content.colorspace.%s.%s" % (color, coord), v)
+					v = round(xyY[j], 4)
+					self.lut3d_set_option("3dlut.content.colorspace.%s.%s" %
+										  (color, coord), v)
 			self.lut3d_update_trc_controls()
 		self.panel.Freeze()
 		sizer = self.lut3d_content_colorspace_red_x.ContainingSizer
@@ -443,7 +454,8 @@ class LUT3DFrame(BaseFrame):
 
 	def lut3d_content_colorspace_xy_handler(self, event):
 		option = event.GetEventObject().Name.replace("_", ".")[5:]
-		setcfg("3dlut" + option, event.GetEventObject().GetValue())
+		self.lut3d_set_option("3dlut" + option,
+							  event.GetEventObject().GetValue())
 		self.lut3d_update_trc_controls()
 	
 	def lut3d_create_consumer(self, result=None):
@@ -727,8 +739,11 @@ class LUT3DFrame(BaseFrame):
 														"ColorLookupTable.fx")
 							with open(clut_fx_path, "wb") as clut_fx_file:
 								clut_fx_file.write(clut_fx)
-							with open(reshade_fx_path, "wb") as reshade_fx_file:
-								reshade_fx_file.write(reshade_fx)
+							if not os.path.isdir(os.path.join(dst_dir,
+															  "reshade-shaders")):
+								# ReShade < 3.0
+								with open(reshade_fx_path, "wb") as reshade_fx_file:
+									reshade_fx_file.write(reshade_fx)
 					for src_path, dst_path in zip(src_paths, dst_paths):
 						shutil.copyfile(src_path, dst_path)
 					return
@@ -756,6 +771,12 @@ class LUT3DFrame(BaseFrame):
 				trc_gamma = getcfg("3dlut.trc_gamma")
 		else:
 			trc_gamma = None
+		XYZwp = None
+		if not isinstance(self, LUT3DFrame):
+			x = getcfg("3dlut.whitepoint.x", False)
+			y = getcfg("3dlut.whitepoint.y", False)
+			if x and y:
+				XYZwp = colormath.xyY2XYZ(x, y)
 		trc_gamma_type = getcfg("3dlut.trc_gamma_type")
 		outoffset = getcfg("3dlut.trc_output_offset")
 		intent = getcfg("3dlut.rendering_intent")
@@ -790,7 +811,9 @@ class LUT3DFrame(BaseFrame):
 									 apply_black_offset=apply_black_offset,
 									 use_b2a=use_b2a, white_cdm2=white_cdm2,
 									 maxcll=maxcll,
-									 content_rgb_space=content_rgb_space)
+									 content_rgb_space=content_rgb_space,
+									 hdr_display=getcfg("3dlut.hdr_display"),
+									 XYZwp=XYZwp)
 		except Exception, exception:
 			return exception
 		return True
@@ -863,8 +886,14 @@ class LUT3DFrame(BaseFrame):
 			if getattr(self, "lut3dframe", None):
 				self.lut3dframe.lut3d_update_shared_controls()
 			return
-		elif self.Parent:
-			self.Parent.lut3d_update_shared_controls()
+		else:
+			self.panel.Freeze()
+			self.lut3d_show_hdr_display_control()
+			self.panel.Layout()
+			self.panel.Thaw()
+			self.update_layout()
+			if self.Parent:
+				self.Parent.lut3d_update_shared_controls()
 		self.lut3d_create_btn.Enable(format != "madVR" or
 									 self.output_profile_ctrl.IsShown())
 	
@@ -972,9 +1001,9 @@ class LUT3DFrame(BaseFrame):
 					(which == "abstract" and
 					 (profile.profileClass != "abst" or profile.colorSpace
 					  not in ("Lab", "XYZ")))):
-					show_result_dialog(NotImplementedError(lang.getstr("profile.unsupported", 
-																	   (profile.profileClass, 
-																		profile.colorSpace))),
+					show_result_dialog(Error(lang.getstr("profile.unsupported", 
+														 (profile.profileClass, 
+														  profile.colorSpace))),
 									   parent=self)
 				else:
 					if profile.profileClass == "link":
@@ -1180,6 +1209,11 @@ class LUT3DFrame(BaseFrame):
 				self.lut3d_formats_ab[i] = format
 				self.lut3d_formats_ba[format] = i
 				i += 1
+
+		self.lut3d_hdr_display_ctrl.SetItems([lang.getstr(item)
+											  for item in
+											  ("3dlut.format.madVR.hdr_to_sdr",
+											   "3dlut.format.madVR.hdr")])
 		
 		self.lut3d_size_ab = {}
 		self.lut3d_size_ba = {}
@@ -1210,12 +1244,6 @@ class LUT3DFrame(BaseFrame):
 			encodings = list(video_encodings)
 			config.defaults["3dlut.encoding.input"] = "n"
 			config.defaults["3dlut.encoding.output"] = "n"
-			if format == "eeColor":
-				# As eeColor usually needs same input & output encoding,
-				# and xvYCC output encoding is not supported generally,
-				# remove xvYCC input encoding choices for eeColor
-				encodings.remove("x")
-				encodings.remove("X")
 		if (self.worker.argyll_version >= [1, 7] and
 			self.worker.argyll_version != [1, 7, 0, "_beta"]):
 			# Argyll 1.7 beta 3 (2015-04-02) added clip WTW on input TV encoding
@@ -1278,6 +1306,7 @@ class LUT3DFrame(BaseFrame):
 			format = "cube"
 			setcfg("3dlut.format", format)
 		self.lut3d_format_ctrl.SetSelection(self.lut3d_formats_ba[getcfg("3dlut.format")])
+		self.lut3d_hdr_display_ctrl.SetSelection(getcfg("3dlut.hdr_display"))
 		self.lut3d_size_ctrl.SetSelection(self.lut3d_size_ba[getcfg("3dlut.size")])
 		self.lut3d_enable_size_controls()
 		self.lut3d_bitdepth_input_ctrl.SetSelection(self.lut3d_bitdepth_ba[getcfg("3dlut.bitdepth.input")])
@@ -1297,12 +1326,15 @@ class LUT3DFrame(BaseFrame):
 			getcfg("3dlut.trc_output_offset") == 0 and
 			getcfg("3dlut.trc_gamma") == 2.4):
 			self.lut3d_trc_ctrl.SetSelection(1)  # BT.1886
+			setcfg("3dlut.trc", "bt1886")
 		elif (getcfg("3dlut.trc_gamma_type") == "b" and
 			getcfg("3dlut.trc_output_offset") == 1 and
 			getcfg("3dlut.trc_gamma") == 2.2):
 			self.lut3d_trc_ctrl.SetSelection(0)  # Pure power gamma 2.2
+			setcfg("3dlut.trc", "gamma2.2")
 		else:
 			self.lut3d_trc_ctrl.SetSelection(4)  # Custom
+			setcfg("3dlut.trc", "customgamma")
 
 	def lut3d_update_trc_controls(self):
 		self.lut3d_update_trc_control()
@@ -1350,7 +1382,7 @@ class LUT3DFrame(BaseFrame):
 			self.input_profile.tags.bTRC and
 			isinstance(self.input_profile.tags.rTRC,
 					   ICCP.CurveType)):
-			tf = self.input_profile.tags.rTRC.get_transfer_function()
+			tf = self.input_profile.tags.rTRC.get_transfer_function(outoffset=1.0)
 			if (getcfg("3dlut.input.profile") !=
 				self.input_profile.fileName):
 				# Use BT.1886 gamma mapping for SMPTE 240M /
@@ -1389,7 +1421,10 @@ class LUT3DFrame(BaseFrame):
 						 ICCP.LUT16Type) and
 			  isinstance(self.input_profile.tags.get("A2B2", ICCP.LUT16Type()),
 						 ICCP.LUT16Type)):
-			self.lut3d_trc_apply_black_offset_ctrl.Enable()
+			self.lut3d_trc_apply_black_offset_ctrl.Enable(self.XYZbpin !=
+														  self.XYZbpout)
+			if self.XYZbpin == self.XYZbpout:
+				setcfg("3dlut.apply_black_offset", 0)
 		else:
 			self.lut3d_trc_apply_black_offset_ctrl.Disable()
 			setcfg("3dlut.apply_black_offset", 0)
@@ -1419,6 +1454,12 @@ class LUT3DFrame(BaseFrame):
 		if not frozen:
 			self.Thaw()
 
+	def lut3d_show_hdr_display_control(self):
+		self.lut3d_hdr_display_ctrl.Show((getcfg("3dlut.apply_trc") or
+								not hasattr(self, "lut3d_trc_apply_none_ctrl")) and
+							   getcfg("3dlut.trc").startswith("smpte2084") and
+							   getcfg("3dlut.format") == "madVR")
+
 	def lut3d_show_trc_controls(self, show=True):
 		self.panel.Freeze()
 		show = show and self.worker.argyll_version >= [1, 6]
@@ -1454,6 +1495,7 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_hdr_peak_luminance_label.Show(smpte2084)
 		self.lut3d_hdr_peak_luminance_ctrl.Show(smpte2084)
 		self.lut3d_hdr_peak_luminance_ctrl_label.Show(smpte2084)
+		self.lut3d_show_hdr_display_control()
 		self.panel.Layout()
 		self.panel.Thaw()
 		if isinstance(self, LUT3DFrame):
@@ -1475,8 +1517,7 @@ class LUT3DFrame(BaseFrame):
 		self.encoding_input_ctrl.SetSelection(self.encoding_input_ba[getcfg("3dlut.encoding.input")])
 		self.encoding_input_ctrl.Enable(self.encoding_input_ctrl.Count > 1)
 		self.encoding_output_ctrl.SetSelection(self.encoding_output_ba[getcfg("3dlut.encoding.output")])
-		self.encoding_output_ctrl.Enable(getcfg("3dlut.format") not in ("eeColor",
-																		"madVR"))
+		self.encoding_output_ctrl.Enable(getcfg("3dlut.format") != "madVR")
 	
 	def lut3d_enable_size_controls(self):
 		self.lut3d_size_ctrl.Enable(getcfg("3dlut.format")
